angular.module('flashy.CardGridController', ['ui.router', 'ngAnimate', 'ngWebSocket', 'flashy.DeckFactory']).CardGridController =
    function($scope, $rootScope, $state, $http, $window, $timeout, $stateParams, $websocket, $interval, UserService, Flashcard, Deck) {
        sectionId = parseInt($stateParams.sectionId);
        if (!UserService.isInSection(sectionId)) $state.go('addclass');
        $scope.cards = []; // all cards
        $scope.cardCols = null; // organized data
        $scope.affectedCards = []; // cards affected by a position change move
        $scope.numCols = 0;
        $scope.height = 0; // height of a card
        $scope.section = $rootScope.SectionResource.get({sectionId: sectionId});
        $scope.currentEditingCard = null;
        $scope.deck = new Deck(sectionId, {
            cardHideCallback: function(card) {
                $scope.hideCardFromGrid(card);
                if (!card.isReported && !card.isEdited)
                    Materialize.toast('Card hidden! If the card was spammy or abusive, you may also ' +
                        '<a class="btn-flat cyan-text" onclick="card.report()">report</a>', 4000);
            },
            cardPullCallback: $scope.cardPullCallback,
        });

        $('.edit-modal-trigger').leanModal({
            dismissible: true, // Modal can be dismissed by clicking outside of the modal
            opacity: .5, // Opacity of modal background
            in_duration: 300, // Transition in duration
            out_duration: 200, // Transition out duration
            ready: function() {
                console.log('hi');
                $('#edit-card-input').html($scope.currentEditingCard);
                $('#edit-card-input').get(0).focus();
            }, // Callback for Modal open
            complete: function() {
                $scope.currentEditingCard = null;
                console.log('EDIT MODAL CLOSED');

            } // Callback for Modal close
        });

        Flashcard.linkDeck($scope.deck);

        $scope.moveQueue = [];

        $scope.procQueue;
        $scope.addMove = function(m) {
            if (!$scope.moveQueue) return m.go();
            $scope.moveQueue.push(m);
        };

        $scope.showGrid = false;
        //$scope.moveQueue = []; // queue of flashcard objects
        $rootScope.currentSection = $scope.section;

        $scope.refreshColumnWidth = function() {
            avail = $window.innerWidth - 17;
            width = Math.floor(avail / Math.floor(avail / 250));
            $('.cardColumn').css({
                width: width + 'px'
            });
            $('.cardColumn .card.flashy').css({
                width: width - 12 + 'px',
                'font-size': 100 * width / 250 + '%',
                height: (width * 3 / 5) + 'px'
            });
            /*
             $('.cardColumn .card.flashy i.small').css({
             'font-size': 200 * width / 250 + '%'
             });
             $('.cardColumn .card.flashy i.medium').css({
             'font-size': 400 * width / 250 + '%'
             });*/
            $scope.height = width * 3 / 5;
        };

        $scope.refreshLayout = function() {
            numCols = Math.max(1, Math.floor(($window.innerWidth - 17) / 250));
            // check if we actually need to refresh the whole layout
            if (numCols == $scope.numCols) return $scope.refreshColumnWidth();
            $scope.numCols = numCols;
            console.log('refreshing layout for ' + numCols + ' columns');
            $scope.cardCols = [];
            var cols = [];
            for (var i = 0; i < numCols; i++) cols.push([]);
            var n = 0;
            $scope.cards.forEach(function(card, j) {
                card.colNum = n++ % numCols;
                cols[card.colNum].push(card);
            });
            for (i in cols) $scope.updateColRanks(cols[i]);
            $scope.cardCols = cols;
            return $timeout($scope.refreshColumnWidth);
        };

        angular.element($window).bind('resize', $scope.refreshLayout);

        $scope.addCardToGrid = function(card) {
            console.log('adding card to grid', card);
            var colNum = 0;
            var lowestCol = $scope.cardCols[0];
            var lowestColNum = 0;
            while (colNum < $scope.numCols) {
                if ($scope.cardCols[colNum].length == 0) {
                    lowestCol = $scope.cardCols[colNum];
                    lowestColNum = colNum;
                    break;
                } else if ($scope.cardCols[colNum].length < lowestCol.length) {
                    lowestCol = $scope.cardCols[colNum];
                    lowestColNum = colNum;
                    lowestColLen = $scope.cardCols[colNum].length;
                }
                colNum++;
            }
            card.colNum = lowestColNum;
            $scope.cards.push(card);
            $scope.cardCols[lowestColNum].unshift(card);

            $scope.updateColRanks(lowestCol);
            console.log($scope.cardCols);
            $timeout($scope.refreshColumnWidth);
        };

        $scope.updateColRanks = function(col) {
            for (i in col)
                col[i].colRank = parseInt(i);
        };

        $scope.hideCardFromGrid = function(card) {
            console.log('hiding', card);
            if (typeof card.colNum === 'undefined') return;
            $scope.cardCols[card.colNum].splice(card.colRank, 1);
            card.colNum = undefined;
            card.colRank = undefined;
            $scope.updateColRanks($scope.cardCols[card.colNum]);
            console.log($scope.cardCols);
        };

        $scope.$on('$destroy', function() {
            $scope.deck.cleanup();
            Flashcard.cleanup();
            $rootScope.currentSection = {};
            $(document).off('keydown');
        });

        $scope.editCard = function(card) {
            $scope.currentEditingCard = card;
            $scope.editCardFormattedText = card.formatted_text;
            console.log(card);
            $('#editModal').openModal(card.editModalOptions);
        };

        $scope.saveEditChanges = function() {
            if (!$scope.currentEditingCard) return;
            if ($('#edit-card-input').text().length == 0) return;
            var myCard = {
                'text': $('#edit-card-input').text(),
                'mask': $scope.editCardBlanks,
            };
            $scope.currentEditingCard.isEdited = true;
            if (myCard.text != '')
                $http.patch('/api/flashcards/' + $scope.currentEditingCard.id + '/', myCard).
                    success(function(data) {
                        console.log('flashcard edited: ' + myCard.text);
                    });
            $scope.currentEditingCard = null;
            $('#editModal').closeModal();
        };

        $scope.discardEditChanges = function() {
            $scope.currentEditingCard = null;
            $('#editModal').closeModal();
        };

        $scope.refreshEditCardInput = function() {
            $scope.editCardText = $('#edit-card-input').text();

            $scope.submit_enabled = $scope.editCardText.length >= 5 && $scope.editCardText.length <= 160;

            var i = 0;
            $scope.editCardBlanks = [];
            $('#edit-card-input')[0].childNodes.forEach(function(node) {
                node = $(node)[0];
                if (node.tagName == 'B') {
                    var text = $(node).text();
                    var leftspaces = 0, rightspaces = 0;
                    // awful way to find the first non-space character from the left or the right. thanks.js
                    while (text[leftspaces] == ' ' || text[leftspaces] == '\xa0') leftspaces++;
                    while (text[text.length - 1 - rightspaces] == ' ' || text[text.length - 1 - rightspaces] == '\xa0') rightspaces++;
                    console.log(leftspaces, text.length);
                    if (leftspaces != text.length) $scope.editCardBlanks.push([i + leftspaces, i + text.length - rightspaces]);
                    i += text.length;
                } else if (!node.data) {
                    i += $(node).text().length;
                } else {
                    i += node.data.length;
                }
            });
            $scope.editCardBlanks.sort(function(a, b) {
                return a[0] - b[0];
            });
            i = 0;
            newtext = '';
            $scope.editCardBlanks.forEach(function(blank) {
                newtext += $scope.editCardText.slice(i, blank[0]);
                newtext += '<b>' + $scope.editCardText.slice(blank[0], blank[1]) + '</b>';
            });
        };

        $('div[contenteditable]').keydown(function(e) {
            if (e.keyCode === 13) {
                $scope.saveEditChanges();
                e.preventDefault();
                return false;
            }
        });

        return $scope.deck.deckPromise;
    };
