angular.module('flashy.DeckFactory', ['ui.router', 'flashy.FlashcardFactory', 'ngWebSocket']).
    factory('Deck', function ($http, $rootScope, $state, $websocket, Flashcard, UserService) {

        var Deck = function (sectionId, callbacks) {
            if (!UserService.isInSection(sectionId)) {
                console.log('user is not enrolled in ' + sectionId);
                $state.go('addclass');
            }
            obj = this;
            this.cards = [];
            this.section = $rootScope.SectionResource.get({sectionId: sectionId});

            this.ws = $websocket($rootScope.ws_host + '/ws/deck/' + sectionId + '?subscribe-user');
            this.contains = function (id) {
                return this.cards[id];
            };

            this.ws.onMessage(function (message) {
                data = JSON.parse(message.data);
                console.log('message', data);
                if (data.event_type == 'card_pulled') {
                    if (obj.cards[data.flashcard.id]) console.log("Pulled a card already in our deck?");
                    card = new Flashcard(data.flashcard);
                    if (data.flashcard.is_authored_by_user) card.is_authored_by_user = true;
                    obj.cards[card.id] = card;
                    if (callbacks.cardPullCallback) callbacks.cardPullCallback(card);
                }
                if (data.event_type == 'card_unpulled') {
                    if (!obj.cards[data.flashcard.id]) return console.log("Unpulled a card not in our deck?");
                    card = new Flashcard(data.flashcard);
                    obj.cards[card.id] = undefined;
                    if (callbacks.cardUnpullCallback) callbacks.cardUnpullCallback(card);
                }
                if (data.event_type == 'card_hidden') {
                    card = new Flashcard(data.flashcard.id);
                    console.log(card);
                    card.is_hidden = true;
                    if (callbacks.cardHideCallback) callbacks.cardHideCallback(card);
                }
                if (data.event_type == 'card_unhidden') {
                    if (obj.cards[data.flashcard.id]) return;
                    card = new Flashcard(data.flashcard);
                    card.is_hidden = false;
                    if (callbacks.cardUnhideCallback) callbacks.cardUnhideCallback(card);
                }
                if (data.event_type == 'card_fixed') {
                    card = obj.cards[data.flashcard.id].edit(data.flashcard);
                    console.log("fixed card", obj.cards[card.id]);
                    if (callbacks.cardFixedCallback) callbacks.cardFixedCallback(card);
                }
            });
            this.deckPromise = $http.get('/api/sections/' + sectionId + '/deck/').success(function (data) {
                for (i in data) {
                    obj.cards[data[i].id] = new Flashcard(data[i], obj);
                    //obj.cards[data[i].id].is_authored_by_user = data.is_authored_by_user;
                }

                console.log("got user's deck", data);
            });
            this.cleanup = function () {
                this.ws.close();
            };
            this.forEach = this.cards.forEach;
        };

        return Deck;
    });