angular.module('flashy.UserService', ['ui.router']).
    service('UserService', function($rootScope, $http, $q) {
        var deferred = $q.defer();
        var _user = false;
        var login = function(data) {
            if (data.locked) {
                $rootScope.UserService.showLockedMessage();
                return deferred.resolve(data);
            }
            if (!data.is_confirmed) {
                Materialize.toast('Please verify your email address! ' +
                    '<a class="btn-flat cyan-text" onclick="rootscope.UserService.resendConfirmationEmail()">' +
                    'Resend Verification Email</a>', 4000);
            }
            _user = data;
            _user.sectionIdList = _user.sections.map(function(x) {
                return x.id;
            });
            deferred.resolve(data);
        };
        this.login = login;
        this.startAuth = function() {
            return $http.get('/api/me/').success(function(data) {
                console.log('user is logged in!');
                login(data);
            }).error(function(data) {
                console.log(data);
                console.log('not logged in yet: ' + data.detail);
                _user = {email: false};
                deferred.resolve(_user);
            });
        };

        this.isResolved = function() {
            return !!_user;
        };
        this.getUserData = function() {
            if (this.isResolved()) return _user;
            else return deferred.promise;
        };
        this.hasVerifiedEmail = function() {
            return this.isResolved() && _user.is_confirmed;
        };

        this.logout = function($state) {
            $http.post('/api/logout/').success(function() {
                if (!_user.locked)Materialize.toast('Logged out!', 1000);
            });
            _user = false;
            deferred.resolve({});
            $state.go('login');
        };
        this.addClass = function(section) {
            _user.sections.push(section);
            _user.sectionIdList.push(section.id);
        };
        this.dropClass = function(sectionId) {
            for (i in _user.sections) {
                if (_user.sections[i].id == sectionId) _user.sections.splice(i, 1);
            }
            for (i in _user.sectionIdList) {
                if (_user.sectionIdList[i] == sectionId) _user.sectionIdList.splice(i, 1);
            }
        };
        this.isLoggedIn = function() {
            rv = this.isResolved() && _user.email;
            return rv;
        };
        this.isInSection = function(sectionId) {
            return (_user.sectionIdList.indexOf(sectionId) >= 0);
        };
        this.redirectToDefaultState = function($state) {
            console.log('redirecting user to their default state');
            // if the user isn't logged in, log in!
            if (!this.isLoggedIn()) return $state.go('login');
            // if the user isn't enrolled in any sections, go to addclass
            if (!_user.sections.length) return $state.go('addclass');
            last_state = localStorage.getItem('last_state');
            if (last_state) {
                // if there was a last state, get the parameters of the state
                last_state_params = JSON.parse(localStorage.getItem('last_state_params'));
                if (last_state_params.sectionId && this.authorizedFor(last_state, last_state_params)) {
                    // if we're authorized to access that state with those parameters, go there
                    return $state.go(last_state, JSON.parse(localStorage.getItem('last_state_params')));
                }
            }
            $state.go('feed', {sectionId: _user.sections[0].id});
        };
        this.authorizedFor = function(state, stateParams) {
            if (['feed', 'deck', 'cardlist'].indexOf(state.name) >= 0) {
                if (_user.sectionIdList.indexOf(parseInt(stateParams.sectionId)) < 0) {
                    return false;
                }
            }
            return true;
        };
        this.showLockedMessage = function() {
            Materialize.toast('You must verify your email address before continuing.' +
                '<a class="btn-flat cyan-text" onclick="rootscope.UserService.resendConfirmationEmail()">' +
                'Resend Verification Email</a>', 4000);
        };
        this.noAuthRequired = function(state) {
            return ['verifyemail', 'login'].indexOf(state.name) >= 0;
        };
        this.resendConfirmationEmail = function() {
            console.log('Requesting resend of confirmation email');
            $http.post('/api/resend_confirmation_email/').success(function() {
                Materialize.toast('Resent confirmation email! Check your spam folder too.', 4000);
            });
        };
        this.startAuth();
    });
